@@ -5,8 +5,14 @@
 import numpy as np
 import pytest
 
-from pde import CartesianGrid, PolarSymGrid, ScalarField, Tensor2Field, VectorField
-from pde.grids.operators import polar_sym as ops
+from pde import (
+    CartesianGrid,
+    PolarSymGrid,
+    ScalarField,
+    Tensor2Field,
+    VectorField,
+    solve_poisson_equation,
+)
 
 
 def test_findiff():
@@ -30,26 +36,25 @@ def test_findiff():
     np.testing.assert_allclose(div.data, [5, 17 / 3, 2 + 4 / r2])
 
 
-def test_conservative_laplace():
+def test_conservative_laplace_polar():
     """test and compare the two implementation of the laplace operator"""
     grid = PolarSymGrid(1.5, 8)
     f = ScalarField.random_uniform(grid)
 
-    bcs = grid.get_boundary_conditions("natural")
-    lap = ops.make_laplace(bcs)
-    np.testing.assert_allclose(f.apply(lap).integral, 0, atol=1e-12)
+    res = f.laplace("natural")
+    np.testing.assert_allclose(res.integral, 0, atol=1e-12)
 
 
 @pytest.mark.parametrize(
-    "make_op,field,rank",
+    "op_name,field",
     [
-        (ops.make_laplace, ScalarField, 0),
-        (ops.make_divergence, VectorField, 0),
-        (ops.make_gradient, ScalarField, 0),
-        (ops.make_tensor_divergence, Tensor2Field, 1),
+        ("laplace", ScalarField),
+        ("divergence", VectorField),
+        ("gradient", ScalarField),
+        ("tensor_divergence", Tensor2Field),
     ],
 )
-def test_small_annulus(make_op, field, rank):
+def test_small_annulus_polar(op_name, field):
     """test whether a small annulus gives the same result as a sphere"""
     grids = [
         PolarSymGrid((0, 1), 8),
@@ -59,13 +64,13 @@ def test_small_annulus(make_op, field, rank):
 
     f = field.random_uniform(grids[0])
 
-    res = [make_op(g.get_boundary_conditions(rank=rank))(f.data) for g in grids]
+    res = [field(g, data=f.data)._apply_operator(op_name, "natural") for g in grids]
 
-    np.testing.assert_almost_equal(res[0], res[1], decimal=5)
-    assert np.linalg.norm(res[0] - res[2]) > 1e-3
+    np.testing.assert_almost_equal(res[0].data, res[1].data, decimal=5)
+    assert np.linalg.norm(res[0].data - res[2].data) > 1e-3
 
 
-def test_grid_laplace():
+def test_grid_laplace_polar():
     """test the polar implementation of the laplace operator"""
     grid_sph = PolarSymGrid(7, 8)
     grid_cart = CartesianGrid([[-5, 5], [-5, 5]], [12, 11])
@@ -81,11 +86,11 @@ def test_grid_laplace():
     np.testing.assert_allclose(b_1d_2.data[i, i], b_2d.data[i, i], rtol=0.2, atol=0.2)
 
 
-@pytest.mark.parametrize("r_inner", (0, 1))
-def test_gradient_squared(r_inner):
+@pytest.mark.parametrize("r_inner", (0, 2 * np.pi))
+def test_gradient_squared_polar(r_inner):
     """compare gradient squared operator"""
-    grid = PolarSymGrid((r_inner, 5), 64)
-    field = ScalarField.random_harmonic(grid, modes=1)
+    grid = PolarSymGrid((r_inner, 4 * np.pi), 32)
+    field = ScalarField.from_expression(grid, "cos(r)")
     s1 = field.gradient("natural").to_scalar("squared_sum")
     s2 = field.gradient_squared("natural", central=True)
     np.testing.assert_allclose(s1.data, s2.data, rtol=0.1, atol=0.1)
@@ -94,47 +99,31 @@ def test_gradient_squared(r_inner):
     assert not np.array_equal(s2.data, s3.data)
 
 
-def test_grid_div_grad():
+def test_grid_div_grad_polar():
     """compare div grad to laplacian for polar grids"""
     grid = PolarSymGrid(2 * np.pi, 16)
-    r = grid.axes_coords[0]
-    arr = np.cos(r)
+    field = ScalarField.from_expression(grid, "cos(r)")
 
-    laplace = grid.get_operator("laplace", "derivative")
-    grad = grid.get_operator("gradient", "derivative")
-    div = grid.get_operator("divergence", "value")
-    a = laplace(arr)
-    b = div(grad(arr))
-    res = -np.sin(r) / r - np.cos(r)
+    a = field.laplace("derivative")
+    b = field.gradient("derivative").divergence("value")
+    res = ScalarField.from_expression(grid, "-sin(r) / r - cos(r)")
 
     # do not test the radial boundary points
-    np.testing.assert_allclose(a[1:-1], res[1:-1], rtol=0.1, atol=0.1)
-    np.testing.assert_allclose(b[1:-1], res[1:-1], rtol=0.1, atol=0.1)
+    np.testing.assert_allclose(a.data[1:-1], res.data[1:-1], rtol=0.1, atol=0.1)
+    np.testing.assert_allclose(b.data[1:-1], res.data[1:-1], rtol=0.1, atol=0.1)
 
 
 def test_poisson_solver_polar():
     """test the poisson solver on Polar grids"""
-    grid = PolarSymGrid(4, 8)
-    for bc_val in ["natural", {"value": 1}]:
-        bcs = grid.get_boundary_conditions(bc_val)
-        poisson = grid.get_operator("poisson_solver", bcs)
-        laplace = grid.get_operator("laplace", bcs)
-
-        d = np.random.random(grid.shape)
-        d -= ScalarField(grid, d).average  # balance the right hand side
-        res = laplace(poisson(d))
-        np.testing.assert_allclose(res, d, rtol=1e-6, err_msg=f"bcs = {bc_val}")
-
-    grid = PolarSymGrid([2, 4], 8)
-    for bc_val in ["natural", {"value": 1}]:
-        bcs = grid.get_boundary_conditions(bc_val)
-        poisson = grid.get_operator("poisson_solver", bcs)
-        laplace = grid.get_operator("laplace", bcs)
-
-        d = np.random.random(grid.shape)
-        d -= ScalarField(grid, d).average  # balance the right hand side
-        res = laplace(poisson(d))
-        np.testing.assert_allclose(res, d, rtol=1e-6, err_msg=f"bcs = {bc_val}")
+    for grid in [PolarSymGrid(4, 8), PolarSymGrid([2, 4], 8)]:
+        for bc_val in ["natural", {"value": 1}]:
+            bcs = grid.get_boundary_conditions(bc_val)
+            d = ScalarField.random_uniform(grid)
+            d -= d.average  # balance the right hand side
+            sol = solve_poisson_equation(d, bcs)
+            test = sol.laplace(bcs)
+            msg = f"grid={grid}, bcs={bc_val}"
+            np.testing.assert_allclose(test.data, d.data, err_msg=msg, rtol=1e-6)
 
 
 def test_examples_scalar_polar():
