@@ -290,7 +290,7 @@ class PDE(PDEBase):
                 self._logger.info(msg, bc, func, var)
 
                 try:
-                    ops[func] = state.grid.get_operator(func, bc=bc)
+                    ops[func] = state.grid.make_operator(func, bc=bc)
                 except BCDataError:
                     # wrong data was supplied for the boundary condition
                     raise
@@ -374,7 +374,7 @@ class PDE(PDEBase):
         cache["state_attributes"] = state.attributes
         return cache
 
-    def evolution_rate(self, state: FieldBase, t: float = 0) -> FieldBase:
+    def evolution_rate(self, state: FieldBase, t: float = 0.0) -> FieldBase:
         """evaluate the right hand side of the PDE
 
         Args:
@@ -389,22 +389,25 @@ class PDE(PDEBase):
         """
         cache = self._prepare_cache(state, backend="numpy")
 
+        # create an empty copy of the current field
+        result = state.copy()
+
+        # fill it with data
         if isinstance(state, DataFieldBase):
             # state is a single field
-            rhs = cache["rhs_funcs"][0]
-            return state.copy(data=rhs(state.data, t))
+            result.data[:] = cache["rhs_funcs"][0](state.data, t)
 
         elif isinstance(state, FieldCollection):
             # state is a collection of fields
-            result = state.copy()
             for i in range(len(state)):
                 data_tpl = cache["get_data_tuple"](state.data)
-                result[i].data = cache["rhs_funcs"][i](*data_tpl, t)
-            return result
+                result[i].data[:] = cache["rhs_funcs"][i](*data_tpl, t)  # type: ignore
 
         else:
             raise TypeError(f"Unsupported field {state.__class__.__name__}")
 
+        return result
+
     def _make_pde_rhs_numba_coll(
         self, state: FieldCollection, cache: Dict[str, Any]
     ) -> Callable[[np.ndarray, float], np.ndarray]:
